/*
 * $Id: common.c,v 1.4 2003/03/06 14:05:34 tsavola Exp $
 * Timo Savola <timo.savola@movial.fi>
 */

#include <stdio.h>
#include <stdlib.h>
#include <ctype.h>
#include <string.h>
#include <unistd.h>
#include <fcntl.h>

/**
 * Sets a file descriptor non-blocking.
 */
int set_nonblocking(int fd)
{
	int val;

	val = fcntl(fd, F_GETFL, 0);
	if (val < 0) {
		return -1;
	}

	if (val & O_NONBLOCK) {
		return 0;
	}

	return fcntl(fd, F_SETFL, val | O_NONBLOCK);
}

/**
 * Gets a neat progname.
 */
char *get_progname(char *name)
{
	char *p;

	p = strchr(name, '/');
	if (p && strlen(p) >= 2) {
		return &p[1];
	} else {
		return name;
	}
}

/**
 * Frees all components of a null-terminated vector and the vector itself.
 * @param vec tor
 */
void free_vec(void **vec)
{
	void **p;

	for (p = vec; *p; p++) {
		free(*p);
	}

	free(vec);
}

int read_line(FILE *file,
	      char *buf,
	      int size)
{
	int comm = 0;
	int c, i = 0;

	if (feof(file)) {
		return -1;
	}

	while (1) {
		c = fgetc(file);
		if (c == EOF || c == '\n') {
			break;
		}
		if (c == '#') {
			comm = 1;
		}
		if (!comm && i < size - 1) {
			*buf++ = c;
			i++;
		}
	}
	*buf = '\0';

	return i;
}

char *skip_spaces(char *buf)
{
	while (*buf && isspace(*buf)) {
		buf++;
	}

	return buf;
}

char *find_space(char *buf)
{
	while (*buf && !isspace(*buf)) {
		buf++;
	}

	return buf;
}

/**
 * Finds a host from a config file and returns the associated password.
 * If the host has a port defined, it will be stored in the integer pointed
 * to by portp.
 *
 * Example config file entry for client:
 * 192.168.0.4:10000  foobar
 *
 * Example config file entry for daemon:
 * 192.168.0.2        foobar
 *
 * @param filename the config file path
 * @param host name of the remote machine
 * @param portp pointer to a place where the port number may be stored, or NULL
 * @return the password if found, NULL on error
 */
char *find_pwd(const char *filename,
	       const char *host,
	       int *portp)
{
	FILE *file;
	char *pwd = NULL;

	file = fopen(filename, "r");
	if (!file) {
		return NULL;
	}

	while (!pwd && !feof(file)) {
#if 0
		char *_host;
		char *_pwd;
		int i;

		i = fscanf(file, "%as %as", &_host, &_pwd);
		if (i >= 2) {
			char *_port;

			/* Split port string from host string. */
			_port = strchr(_host, ':');
			if (_port && strlen(_port) >= 2) {
				*_port++ = '\0';
			}

			/* Is this the correct host? */
			if (strcmp(host, _host) == 0) {
				pwd = _pwd;
				if (_port && portp) {
					*portp = atoi(_port);
				}
			} else {
				free(_pwd);
			}
		}

		if (i >= 1) {
			free(_host);
		}

		if (i < 2) {
			break;
		}
#else
		char _host[1024 + 1];
		char _pwd[16 + 1];
		int i;

		i = fscanf(file, "%1024s %16s", _host, _pwd);
		if (i >= 2) {
			char *_port;

			/* Split port string from host string. */
			_port = strchr(_host, ':');
			if (_port && strlen(_port) >= 2) {
				*_port++ = '\0';
			}

			/* Is this the correct host? */
			if (strcmp(host, _host) == 0) {
				pwd = strdup(_pwd);
				if (_port && portp) {
					*portp = atoi(_port);
				}
			}
		} else {
			break;
		}
#endif
	}

	fclose(file);

	return pwd;
}
