/*
 * $Id: config.c,v 1.1 2003/03/07 16:21:47 tsavola Exp $
 * Timo Savola <timo.savola@movial.fi>
 */

#include <stdio.h>
#include <stdlib.h>
#include <ctype.h>
#include <string.h>
#include "config.h"
#include "common.h"

extern void error(const char *msg);
extern void error_no(const char *msg);

static char *is_target(char *tgt,
		       char *buf)
{
	int a, b;

	while (1) {
		a = *tgt++;
		b = *buf++;

		if (!a || !b || isspace(b)) {
			return (!a && isspace(b)) ? buf : NULL;
		}

		if (a != b) {
			return NULL;
		}
	}
}

static int read_params(char *buf,
		       config_t *cfg)
{
	char *user, *host, *port, *pwd;

	buf = skip_spaces(buf);
	if (!buf) {
		error("Invalid parameter string");
		return 0;
	}

	host = strchr(buf, '@');
	if (host) {
		user = buf;
		*host++ = '\0';
	} else {
		user = NULL;
		host = buf;
	}

	port = strchr(host, ':');
	if (port) {
		*port++ = '\0';
		buf = port;
	} else {
		port = NULL;
		buf = host;
	}

	buf = find_space(buf);
	if (!buf) {
		error("Invalid parameter string");
		return 0;
	}
	*buf++ = '\0';

	pwd = skip_spaces(buf);
	if (!pwd) {
		error("Invalid parameter string");
		return 0;
	}

	cfg->user = user ? strdup(user) : NULL;
	cfg->host = strdup(host);
	cfg->port = port ? atoi(port) : 0;
	cfg->pwd = strdup(pwd);

	return 1;
}

static int read_opts(FILE *file,
		     char *buf,
		     int size,
		     config_t *cfg)
{
	long pos;
	int memsize, cnt = 0;
	char **p;

	pos = ftell(file);

	while (read_line(file, buf, size) > 0 &&
	       strlen(skip_spaces(buf)) > 0) {
		cnt++;
	}

	fseek(file, pos, SEEK_SET);

	memsize = (cnt + 1) * sizeof (char *);
	cfg->opts = malloc(memsize);
	if (!cfg->opts) {
		error("Out of memory");
		return 0;
	}

	memset(cfg->opts, 0, memsize);

	p = cfg->opts;
	while (cnt-- > 0) {
		read_line(file, buf, size);
		buf = skip_spaces(buf);

		/* Let's be careful. */
		if (strlen(buf) == 0) {
			break;
		}

		*p++ = strdup(buf);
	}

	return 1;
}

int read_config(const char *filename,
		const char *target,
		config_t *cfg)
{
	FILE *file;
	char buf[1024], *buf2;
	int len;
	int ok = 0;

	file = fopen(filename, "r");
	if (!file) {
		return 0;
	}

	while (1) {
		len = read_line(file, buf, sizeof (buf));
		if (len < 0) {
			error("Can't find target");
			break;
		}
		if (len == 0 || isspace(buf[0])) {
			continue;
		}

		buf2 = is_target((char *) target, buf);
		if (buf2) {
			ok = read_params(buf2, cfg) &&
			     read_opts(file, buf, sizeof (buf), cfg);
			break;
		} 
	}

	fclose(file);

	return ok;
}

void init_config(config_t *cfg)
{
	memset(cfg, 0, sizeof (config_t));
}

void free_config(config_t *cfg)
{
	if (cfg->user) {
		free(cfg->user);
	}

	if (cfg->host) {
		free(cfg->host);
	}

	if (cfg->pwd) {
		free(cfg->pwd);
	}

	if (cfg->opts) {
		free_vec((void **) cfg->opts);
	}
}
